#include "dvd.h"
/*+----------------------------------------------------------------------------------------------+*/
/*!
\addtogroup dvd_functions DVD Functions
*/
/*+----------------------------------------------------------------------------------------------+*/
/*!
\ingroup dvd_functions
\brief Checks if the DVD cover is open

\return
   - TRUE when it is open
   - FALSE when it is closed

\todo
   Doesn't work 100%
*/
/*
   Author: Peter (www.console-dev.de)
   Date..: 22nd June 2003
*/
u8 DVD_IsCoverOpen(void)
{
   return (DVD_STATE_COVER_OPEN == (R_DVD_STATUS_2 & 0x000000ff));
}
/*+----------------------------------------------------------------------------------------------+*/
/*!
\ingroup dvd_functions
\brief Get DVD drive information

\param pDVDDriveInfo
   Pointer to a DVD_DRIVEINFO structure which will hold, after calling this function, the required information
*/
/*
   Author: Yursoft (www.yursoft.com)

   25th June 2003 - Modified by Peter (www.console-dev.de)
*/
void DVD_GetDriveInfo(DVD_DRIVEINFO *pDVDDriveInfo)
{
   // Get current offset
   u32 CurOffset = DVD_GetCurrentOffset();

   // Seek to offset 0
	DVD_Seek(0, DVD_SEEK_SET);

   // Read DriveInfo
   DVD_Read((void*)pDVDDriveInfo, sizeof(DVD_DRIVEINFO), 0);

   // Seek to previous offset again
	DVD_Seek(CurOffset, DVD_SEEK_SET);
}
/*+----------------------------------------------------------------------------------------------+*/
/*!
\ingroup dvd_functions
\brief Get value from status register 1 or 2

\param StatusNo
   From which status register you want to get the status from. \n
   This can be one of these:
      - ::DVD_STATUS_1
      - ::DVD_STATUS_2
*/
/*
   Author: Yursoft (www.yursoft.com)

   25th June 2003 - Modified by Peter (www.console-dev.de)
*/
// OK
u32 DVD_GetStatus(u8 StatusNo)
{
   switch(StatusNo)
   {
      case DVD_STATUS_1:
               return R_DVD_STATUS_1;
               break;

      case DVD_STATUS_2:
               return R_DVD_STATUS_2;
               break;

      default:
         ASSERT(0, "DVD_GetStatus: Value must be 1 or 2");
         break;
   }
   return 0;
}
/*+----------------------------------------------------------------------------------------------+*/
/*!
\ingroup dvd_functions
\brief Enable/Disable DVD Audio

\param Value
   What you would like to set, Enable or Disable. \n\n
   This can be one of these defines:
      - ::DVD_AUDIO_ENABLE
      - ::DVD_AUDIO_DISABLE

\todo Not tested yet. Please let me know if it works ...
*/
/*
   Author: Yursoft (www.yursoft.com)

   25th June 2003 - Modified by Peter (www.console-dev.de)
*/
void DVD_SetAudio(u8 Value)
{
   switch(Value)
   {
      case DVD_AUDIO_DISABLE:
               DVD_SetCommands(DVD_COMMAND_AUDIO, 0x00, 0x0000);
               M_DVD_SET_ACTION(0x01);
               break;

      case DVD_AUDIO_ENABLE:
               DVD_SetCommands(DVD_COMMAND_AUDIO, 0x01, 0x0000);
               M_DVD_SET_ACTION(0x01);
               break;

      default:
         ASSERT(0, "DVD_SetAudio: Value must be 0 or 1");
         break;
   }
}
/*+----------------------------------------------------------------------------------------------+*/
/*!
\ingroup dvd_functions
\brief Return DVD ErrorCode

\ret
   Returns the last DVD errorcode.

\todo Not tested yet
*/
/*
   Author: Yursoft (www.yursoft.com)

   25th June 2003 - Modified by Peter (www.console-dev.de)
*/
u32 DVD_GetErrorCode(void)
{
   // Request the error first
   DVD_RequestError();

   // And now return the value from the errorcode register
   return R_DVD_ERRORCODE;
}
/*+----------------------------------------------------------------------------------------------+*/
/*!
\ingroup dvd_functions
\brief Request an error

   Requests the last DVD errorcode. \n
   After calling this function, the errorcode register contains the last errorcode. \n

\todo Not tested yet. Is the describtion ok?
*/
/*
   Author: Yursoft (www.yursoft.com)

   25th June 2003 - Modified by Peter (www.console-dev.de)
*/
void DVD_RequestError(void)
{
   DVD_SetCommands(DVD_COMMAND_REQUESTERROR, 0x00, 0x0000);
	M_DVD_SET_ACTION(0x01);
}
/*+----------------------------------------------------------------------------------------------+*/
/*!
\ingroup dvd_functions
\brief Initialize DVD drive

\bug This function actually makes nothing. I thought it should start the motor
again, but it doesn't. Seems to be incorrect?!? Any infos in this?
*/
/*
   Author: Yursoft (www.yursoft.com)

   25th June 2003 - Modified by Peter (www.console-dev.de)
*/
void DVD_Init(void)
{
   // When dumping status register 1  directly after uploading the
   // dol file, it contains:
   //
   // Status1: 0x0000002A
   // Status2: 0x00000000
   //
   // So,this seems to be the value which is "standard" after uploading
   // the .dol file via psul and PhantasStaryOnline, but it seems not be related
   // to something for initialization. Shouldn't it be a "command" to init the
   // drive, instead of a status register which is normally set by the
   // device itself?!?!?
   M_DVD_SET_STATUS_1(0x2A);
   M_DVD_SET_STATUS_2(0x00);
}
/*+----------------------------------------------------------------------------------------------+*/
/*!
\ingroup dvd_functions
\brief Stop DVD drive

Stops the DVD drive motor
*/
/*
   Author: Yursoft (www.yursoft.com)

   25th June 2003 - Modified by Peter (www.console-dev.de)
*/
// OK
void DVD_Stop(void)
{
   DVD_SetCommands(DVD_COMMAND_STOP, 0x00, 0x0000);
	M_DVD_SET_ACTION(0x01);
}
/*+----------------------------------------------------------------------------------------------+*/
/*!
\ingroup dvd_functions
\brief Read a buffer from an Offset

\param pDstBuffer
   Pointer to the destination-buffer

\param Count
   Total count in bytes to read. (Please see notes)

\param Offset
   Offset you want to start reading from. (Please see notes)

\note
   <b>Offset</b> must be multiple 4! \n
   <b>Count</b> must be multiple 32!
*/
/*
   Author: Yursoft (www.yursoft.com)

   25th June 2003 - Modified by Peter (www.console-dev.de)
*/
// Seems to be ok
void DVD_Read(void *pDstBuffer, u32 Count, u32 Offset)
{
   ASSERT(NULL != pDstBuffer, "DVD_Read: pDstBuffer cannot be NULL");
   ASSERT(0==(Offset%4), "DVD_Read: Offset must be multiple 4");
   ASSERT(0==(Count%32), "DVD_Read: Count must be multiple 32");

   // We don't need to read anything when Count is 0
   if(0==Count) return;

   DVD_SetCommands(DVD_COMMAND_READ, 0x00, 0x0000);
   M_DVD_SET_OFFSET(Offset>>2);
   M_DVD_SET_SRCLENGTH(Count);
   M_DVD_SET_DSTADDRESS(pDstBuffer);
   M_DVD_SET_DSTLENGTH(Count);
   M_DVD_SET_ACTION(0x03);
}
/*+----------------------------------------------------------------------------------------------+*/
/*!
\ingroup dvd_functions
\brief Read a sector

Reads one sector (2048 bytes), stores it in pDstBuffer and sets the dvd position to the next sector.

\param pDstBuffer
   Pointer to the destinationbuffer\n

\param Sector
   The sector you want to read in.

\note
   pDstBuffer must be an allocated buffer of 2048 bytes (1 sector=2048 bytes)
*/
/*
   Author: Yursoft (www.yursoft.com)

   25th June 2003 - Modified by Peter (www.console-dev.de)
*/
// Seems to be ok
void DVD_ReadSector(void *pDstBuffer, u32 Sector)
{
   ASSERT(NULL != pDstBuffer, "DVD_ReadSector: pDstBuffer cannot be NULL");

   DVD_Read(pDstBuffer, DVD_SECTORLENGTH, (Sector*DVD_SECTORLENGTH));
}
/*+----------------------------------------------------------------------------------------------+*/
/*!
\ingroup dvd_functions
\brief Set DVD position

\param Offset
   Signifies the number of bytes to seek from the given whence position and must be multiple 4

\param Whence
   The argument Whence can be:
      - ::DVD_SEEK_SET  :  Seeks from the beginning of the DVD
      - ::DVD_SEEK_CUR  :  Seeks from the current position

Sets the position of the DVD to the given Offset.
*/
/*
   Author: Yursoft (www.yursoft.com)

   25th June 2003 - Modified by Peter (www.console-dev.de)
*/
// Seems to be OK
void DVD_Seek(u32 Offset, u8 Whence)
{
   u32 SeekTo=0;

   ASSERT(0==(Offset%4), "DVD_Seek: Offset must be multiple 4");

   switch(Whence)
   {
      // From beginning //
      case DVD_SEEK_SET:
               SeekTo = Offset;
               break;

      // From current position //
      case DVD_SEEK_CUR:
               SeekTo =  DVD_GetCurrentOffset();
               SeekTo += Offset;
               break;

      default:
         ASSERT(0, "DVD_Seek: Whence must be 0 or 1");
   }
/*
   // Doesn't work here
   DVD_SetCommands(DVD_COMMAND_SEEK, 0x00, 0x0000);
   M_DVD_SET_OFFSET(SeekTo>>2);
   M_DVD_SET_ACTION(0x01);
*/

   // Here is a workaround
   // Since the real "seek" doesn't work (for me),
   // we simply read 0 bytes from the specified offset,
   // which sets the position automaticaly then...
   DVD_SetCommands(DVD_COMMAND_READ, 0x00, 0x0000);
   M_DVD_SET_OFFSET(SeekTo>>2);
   M_DVD_SET_SRCLENGTH(0);
   M_DVD_SET_DSTADDRESS(NULL);
   M_DVD_SET_DSTLENGTH(0);
   M_DVD_SET_ACTION(0x03);
}
/*+----------------------------------------------------------------------------------------------+*/
/*!
\ingroup dvd_functions
\brief Get current DVD position

\ret
   Returns the current DVD position.

\sa
   DVD_Seek() \n
   DVD_SeekSector()
*/
/*
   Author: Peter (www.console-dev.de)
*/
// Seems to be OK
u32 DVD_GetCurrentOffset(void)
{
   return (R_DVD_OFFSET << 2);
}
/*+----------------------------------------------------------------------------------------------+*/
/*!
\ingroup dvd_functions
\brief Get current DVD sector position

\ret
   Returns the current sector position. \n
   CurrentSector = CurrentOffset/SectorLength;

\sa
   DVD_Seek() \n
   DVD_GetCurrentOffset()
*/
/*
   Author: Peter (www.console-dev.de)
*/
// Seems to be OK
u32 DVD_GetCurrentSector(void)
{
   return (DVD_GetCurrentOffset() >> 11);
}
/*+----------------------------------------------------------------------------------------------+*/
/*!
\ingroup dvd_functions
\brief Set DVD position to a given Sector

\param Sector
   Signifies the Sector to seek from the given whence position

\param Whence
   The argument Whence can be:
      - ::DVD_SEEK_SET  :  Seeks from the beginning of the DVD
      - ::DVD_SEEK_CUR  :  Seeks from the current position

Sets the position of the DVD to the given Sector.

\sa
   DVD_Seek() \n
   DVD_GetCurrentOffset()
*/
/*
   Author: Peter (www.console-dev.de)
*/
// Seems to be OK
void DVD_SeekSector(u32 Sector, u8 Whence)
{
   u32 SeekTo=DVD_SEEK_SET;

   switch(Whence)
   {
      // From beginning //
      case DVD_SEEK_SET:
               SeekTo = Sector*DVD_SECTORLENGTH;
               break;

      // From current position //
      case DVD_SEEK_CUR:
               SeekTo =  DVD_GetCurrentOffset();
               SeekTo += (Sector*DVD_SECTORLENGTH);
               break;

      default:
         ASSERT(0, "DVD_SeekSector: Whence must be 0 or 1");
         break;
   }
/*
   // Doesn't work here
   DVD_SetCommands(DVD_COMMAND_SEEK, 0x00, 0x0000);
   M_DVD_SET_OFFSET(SeekTo>>2);
   M_DVD_SET_ACTION(0x01);
*/

   // Here is a workaround
   // Since the real "seek" doesn't work (for me),
   // we simply read 0 bytes from the specified offset,
   // which sets the position automaticaly then...
   DVD_SetCommands(DVD_COMMAND_READ, 0x00, 0x0000);
   M_DVD_SET_OFFSET(SeekTo>>2);
   M_DVD_SET_SRCLENGTH(0);
   M_DVD_SET_DSTADDRESS(NULL);
   M_DVD_SET_DSTLENGTH(0);
   M_DVD_SET_ACTION(0x03);
}
/*+----------------------------------------------------------------------------------------------+*/
/*!
\ingroup dvd_functions
\brief Returns the last command sent

\ret
   Returns the value which is stored in the command register.
   This should be the very last command sent.\n\n

   This is in the following format:
      - 0xAABBCCC \n \n

      Where ... \n
      - AA is the Command. \n
      - BB is the SubCommand. \n
      - CCCC is the SubCommand2. \n
*/
/*
   Author: Peter (www.console-dev.de)
   Date..: 2nd July 2003
*/
u32 DVD_GetLastCommands(void)
{
   return R_DVD_COMMAND;
}
/*+----------------------------------------------------------------------------------------------+*/
/*!
\ingroup dvd_functions
\brief Set DVD command

\param Command
   The command to set.

\sa
   DVD_SetSubCommand() \n
   DVD_SetSubCommand2() \n
*/
/*
   Author: Peter (www.console-dev.de)
   Date..: 2nd July 2003
*/
// OK
void DVD_SetCommand(u8 Command)
{
   u32 Value32;

   // Get current command-register value
   Value32 = R_DVD_COMMAND;

   // Mask it
   Value32 &= 0x00ffffff;

   // Set byte (new command)
   R_DVD_COMMAND = Value32 | (Command << 24);
}
/*+----------------------------------------------------------------------------------------------+*/
/*!
\ingroup dvd_functions
\brief Set DVD sub-command

\param SubCommand
   The sub-command to set

\sa
   DVD_SetCommand() \n
   DVD_SetSubCommand2() \n
*/
/*
   Author: Peter (www.console-dev.de)
   Date..: 2nd July 2003
*/
// OK
void DVD_SetSubCommand(u8 SubCommand)
{
   u32 Value32;

   // Get current command-register value
   Value32 = R_DVD_COMMAND;

   // Mask it
   Value32 &= 0xff00ffff;

   // Set byte (new SubCommand)
   R_DVD_COMMAND = Value32 | (SubCommand << 16);
}
/*+----------------------------------------------------------------------------------------------+*/
/*!
\ingroup dvd_functions
\brief Set DVD sub-command2

\param SubCommand2
   The sub-command2 to set

\sa
   DVD_SetCommand() \n
   DVD_SetSubCommand() \n
*/
/*
   Author: Peter (www.console-dev.de)
   Date..: 2nd July 2003
*/
// OK
void DVD_SetSubCommand2(u16 SubCommand2)
{
   u32 Value32;

   // Get current command-register value
   Value32 = R_DVD_COMMAND;

   // Mask it
   Value32 &= 0xffff0000;

   // Set byte (new SubCommand2)
   R_DVD_COMMAND = Value32 | SubCommand2;
}
/*+----------------------------------------------------------------------------------------------+*/
/*!
\ingroup dvd_functions
\brief Set commands

\param Command
   Command to set

\param SubCommand
   SubCommand to set

\param SubCommand2
   SubCommand2 to set

\sa
   DVD_SetCommand() \n
   DVD_SetSubCommand() \n
   DVD_SetSubCommand2() \n
*/
/*
   Author: Peter (www.console-dev.de)
   Date..: 2nd July 2003
*/
// OK
void DVD_SetCommands(u8 Command, u8 SubCommand, u16 SubCommand2)
{
   DVD_SetCommand       (Command);
   DVD_SetSubCommand    (SubCommand);
   DVD_SetSubCommand2   (SubCommand2);
}
/*+----------------------------------------------------------------------------------------------+*/

/*+----------------------------------------------------------------------------------------------+*/

/*+----------------------------------------------------------------------------------------------+*/

/*+----------------------------------------------------------------------------------------------+*/

/*+----------------------------------------------------------------------------------------------+*/

/*+----------------------------------------------------------------------------------------------+*/

/*+----------------------------------------------------------------------------------------------+*/

/*+----------------------------------------------------------------------------------------------+*/

/*+----------------------------------------------------------------------------------------------+*/
/* END OF FILE */
